<?php
/**
  * Ahead WebSoft Technologies Pvt Ltd.
  * @copyright  Copyright 2006-2026 Ahead WebSoft Technologies Pvt Ltd.
  * @license    https://www.socialengine.com/policies
*/ return array(
  'package' => array(
    'type' => 'external',
    'name' => 'mdetect',
    'version' => '8.1.0',
    'revision' => '$Revision: 7593 $',
    'path' => 'externals/mdetect',
    'repository' => 'socialengine.com',
    'title' => 'mDetect',
    'author' => 'SocialEngine Core',
    'directories' => array(
      'externals/mdetect',
    )
  )
) ?>
